<?php

namespace Drupal\mollie_webform\EventSubscriber;

use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\mollie\Entity\Payment;
use Drupal\mollie\Events\MollieTransactionStatusChangeEvent;
use Drupal\mollie_webform\MollieWebformHelper;
use Drupal\webform\WebformSubmissionInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Class MollieTransactionEventSubscriber.
 */
class MollieTransactionEventSubscriber implements EventSubscriberInterface {

  /**
   * Entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Mollie webform helper.
   *
   * @var \Drupal\mollie_webform\MollieWebformHelper
   */
  protected $mollieWebformHelper;

  /**
   * MollieTransactionEventSubscriber constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   Entity type manager.
   * @param \Drupal\mollie_webform\MollieWebformHelper $mollieWebformHelper
   *  Mollie webform helper.
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager, MollieWebformHelper $mollie_webform_helper) {
    $this->entityTypeManager = $entityTypeManager;
    $this->mollieWebformHelper = $mollie_webform_helper;
  }

  /**
   * Updates the status of the order when the status of a payment has changed.
   *
   * @param \Drupal\mollie\Events\MollieTransactionStatusChangeEvent $event
   *   Event.
   */
  public function updateOrderStatus(MollieTransactionStatusChangeEvent $event): void {
    // Return if the event context is not this module.
    if ($event->getContext() !== 'mollie_webform') {
      return;
    }

    // Default HTTP status code.
    $httpStatusCode = 200;

    // Fetch the transaction.
    try {
      // Get the transaction and check whether it is a payment.
      $transaction = $event->getTransaction();
      if (!($transaction instanceof Payment)) {
        return;
      }

      // Load the webform.
      /** @var \Drupal\webform\WebformSubmissionInterface $submission */
      $submission = $this->entityTypeManager->getStorage('webform_submission')
        ->load($event->getContextId());

      // Check whether the transaction status has changed.
      $element_key = $this->mollieWebformHelper
        ->getMolliePaymentStatusElementNameFromWebformSubmission($submission);
      $current_status = $submission->getElementData($element_key);

      if ($transaction->getStatus() !== $current_status) {
        // Save the new payment status.
        $submission->setElementData($element_key, $transaction->getStatus());
        // We use a regular save() instead of a resave() to be sure hooks are
        // triggered. This allows e.g. to only trigger an email handler for a
        // specific payment status.
        $submission->save();
      }
    } catch (InvalidPluginDefinitionException | PluginNotFoundException | EntityStorageException $e) {
      watchdog_exception('mollie_webform', $e);
      $httpStatusCode = 500;
    }

    // Set the HTTP status code to return to Mollie.
    $event->setHttpStatusCode($httpStatusCode);
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      MollieTransactionStatusChangeEvent::EVENT_NAME => 'updateOrderStatus',
    ];
  }

}
